﻿// asteroid class
//
class Asteroid {
	// define all public and private variables used in the class
	// the movieclip associated with the asteroid
	private var asteroid_mc:MovieClip;
	// the x and y location
	private var x:Number;
	private var y:Number;
	// the velocities along the x and y axis
	private var xv:Number;
	private var yv:Number;
	// the speed at which the asteroid travels in
	private var speed:Number;
	// the speed at which the asteroid rotates
	private var rotationspeed:Number;
	// the original angle of rotation the asteroid is started with
	private var initangle:Number;
	// contains the current angle of rotation for the asteroid
	private var angle:Number;
	// the scale of the asteroid (50%,75%,100%)
	private var scale:Number;
	// the approximate radius of the asteroid
	private var r:Number;
	// the state of the asteroid
	private var state:Number;
	// getter/setter functions
	public function getX():Number {
		return asteroid_mc._x;
	}
	public function setX(x:Number):Void {
		asteroid_mc._x = x;
	}
	public function getY():Number {
		return asteroid_mc._y;
	}
	public function setY(y:Number):Void {
		asteroid_mc._y = y;
	}
	public function getAngle():Number {
		return asteroid_mc._rotation;
	}
	public function setAngle(ang:Number):Void {
		asteroid_mc._rotation = ang;
	}
	public function setScale(sca:Number):Void {
		asteroid_mc._xscale = sca;
		asteroid_mc._yscale = sca;
		scale = sca;
	}
	public function getScale():Number {
		return scale;
	}
	public function getState():Number {
		return state;
	}
	public function setState(st:Number):Void {
		state = st;
	}
	public function getClip():MovieClip {
		return asteroid_mc;
	}
	//
	// define the asteroid object
	// x: starting x location
	// y: starting y location
	// scale: scale of asteroid
	// target: target movie clip to attach asteroids to
	// depth: level the asteroid will be placed on
	public function Asteroid(x:Number, y:Number, scale:Number, target:MovieClip, depth:Number) {
		// pass on all params to the asteroid object properties
		this.x = x;
		this.y = y;
		this.scale = scale;
		// set the asteroid to a random angle
		this.initangle = Math.random()*360;
		// set the current angle to the initial angle
		this.angle = this.initangle;
		// set the rotation speed from -5 to +5 degrees per frame cycle
		this.rotationspeed = (Math.random()-Math.random())*5;
		// set the speed of the asteroid from 2 to 5 vector per frame cycle
		this.speed = (Math.random()*3)+2;
		// atach the correct graphic to the asteroid object
		asteroid_mc = target.attachMovie("asteroidgraphic", "asteroid"+depth, depth);
		// attach a random explode sound to the asteroid, so that each asteroid explosion can sound different 
		asteroid_mc.explode = new Sound(asteroid_mc);
		asteroid_mc.explode.attachSound("EXPLODE"+(int(Math.random()*3)+1));
		// set the initial x,y of the asteroid movie clip
		setX(x);
		setY(y);
		// set the angle of the asteroid movie clip
		setAngle(initangle);
		// set the scale of the asteroid movie clip
		setScale(scale);
		// set the state
		setState(0);
		// now choose a random look for the asteroid, from one of four frames within the asteroid movie clip
		asteroid_mc.gotoAndStop(int(Math.random()*4)+1);
		// based on the look and the scale, determine the approximate radius of the asteroid
		this.r = Math.max(asteroid_mc._width, asteroid_mc._height)/2;
	}
	// function to remove the asteroid movie clip from the stage
	public function kill():Void {
		asteroid_mc.removeMovieClip();
	}
	// function to movie the asteroid along a vector
	public function moveAsteroid():Void {
		// determine velocities along the x and y axis by factoring in speed and angle
		xv = speed*Math.sin(Math.PI/180*initangle);
		yv = -speed*Math.cos(Math.PI/180*initangle);
		// update the x and y positions with the x velocity and y velocitiy
		y += yv;
		x += xv;
		// check to see if the asteroid has gone off the end of the stage
		// this is determined by checking to see if the asteroid + radius is past the edges of the stage
		// if the asteroid runs off the edge of the stage, it placed directly across from its last know position on the stage
		if (x>Stage.width+r) {
			x = -r;
		} else if (x<-r) {
			x = Stage.width+r;
		}
		if (y>Stage.height+r) {
			y = -r;
		} else if (y<-r) {
			y = Stage.height+r;
		}
		// increment the angle of the asteroid by its rotation speed
		angle += rotationspeed;
		// make sure the angle never goes below 0 or goes above 360 degrees
		if (angle<0) {
			angle += 360;
		} else if (angle>360) {
			angle -= 360;
		}
		// update the x, y position of the asteroid movie clip
		setX(x);
		setY(y);
		// set the rotation of the asteroid movie clip
		setAngle(angle);
	}
	// function to destroy the asteroid
	public function destroy():Void {
		// play the explosion sound attached to the asteroid
		asteroid_mc.explode.start();
		// if the asteroid is not already blowing up, then blow it up
		if (state == 0) {
			kill();
		}
	}
}
