﻿// bullet class
//
class Bullet {
	// define all public and private variables used in the class
	// the movie clip associated with the bullet
	private var bullet_mc:MovieClip;
	// the x and y location
	private var x:Number;
	private var y:Number;
	// the velocity along the x and y axis
	private var xv:Number;
	private var yv:Number;
	// the added velocity along the x and y axis, added by the movement of the firing entity
	private var xxv:Number;
	private var yyv:Number;
	// the vector speed at which the bullet travels in
	private var speed:Number;
	// contains the current angle of rotation for the bullet
	private var angle:Number;
	// the state of the asteroid
	private var state:Number;
	// the firing entity of the bullet 
	private var entity:String;
	// getter/setter functions
	public function getX():Number {
		return bullet_mc._x;
	}
	public function setX(x:Number):Void {
		bullet_mc._x = x;
	}
	public function getY():Number {
		return bullet_mc._y;
	}
	public function setY(y:Number):Void {
		bullet_mc._y = y;
	}
	public function getAngle():Number {
		return bullet_mc._rotation;
	}
	public function setAngle(ang:Number):Void {
		bullet_mc._rotation = ang;
	}
	public function getState():Number {
		return state;
	}
	public function setState(st:Number):Void {
		state = st;
	}
	public function getEntity():String {
		return entity;
	}
	public function setEntity(en:String):Void {
		entity = en;
	}
	//
	// define the bullet object
	// entity: firing entity (ufo or spaceShip)
	// x: starting x location
	// y: starting y location
	// xxv: added x velocity from the firing entity
	// yyv: added y velocity from the firing entity
	// speed: speed at which the bullet is launched
	// angle: direction the bullet is launched in
	// target: target movie clip to attach bullets to
	// depth: level the bullet will be placed on
	public function Bullet(entity:String, x:Number, y:Number, xxv:Number, yyv:Number, speed:Number, angle:Number, target:MovieClip, depth:Number) {
		// set up initial velocities
		this.xv = 0;
		this.yv = 0;
		// pass on all params to the bullet object properties
		this.x = x;
		this.y = y;
		this.angle = angle;
		this.xxv = xxv;
		this.yyv = yyv;
		this.speed = speed;
		this.entity = entity;
		// attach correct graphic to the bullet object
		bullet_mc = target.attachMovie("bulletgraphic", "bullet"+depth, depth);
		// attach the firing sound to the bullet object movie clip
		bullet_mc.fire = new Sound(bullet_mc);
		// add a different sound depending on whether the bullet was fired from the ufo or ship 
		bullet_mc.fire.attachSound(entity+"fire");
		// start the fire sound
		bullet_mc.fire.start();
		// set the x and y positions
		setX(x);
		setY(y);
		// set the angle
		setAngle(angle);
		// set its state to 1 (fired)
		setState(1);
	}
	// function to remove to bullet movie clip from the stage
	public function kill():Void {
		bullet_mc.removeMovieClip();
	}
	// function to move the bullet along a vector
	public function moveBullet():Void {
		// determine velocities along the x and y axis by factoring in speed, angle and vector speed of firing entity
		xv = (speed*Math.sin(Math.PI/180*angle))+xxv;
		yv = (-speed*Math.cos(Math.PI/180*angle))+yyv;
		// update the x and y positions with the x velocity and y velocitiy
		y += yv;
		x += xv;
		// update the x, y position of the bullet movie clip
		setX(x);
		setY(y);
	}
}
