﻿// asteroids game class
//
class Game {
	// define the bullet object array
	private var bulletArray:Array;
	// define the asteroid object array
	private var roidArray:Array;
	// counter used to keep track of how many bullets have been fired during the game
	private var bulletCounter:Number;
	// the pause between bullet firing
	private var bulletPause:Number;
	// counter used to keep track of how many asteroids have been created during the game
	private var roidCounter:Number;
	// the maximum allowable pause between bullet firing
	private var maxBulletPause:Number;
	// the deadzone bounding box (x,y) of the ship around which the asteroids can be created 
	private var deadZone:Number;
	// the number of asteroids created at the start of each level/game
	private var numRoids:Number;
	// the target movie clip to which all game assets are attached
	private var target:MovieClip;
	// counter used to measure the amount of time it takes the ufo to explode
	private var ufoCounter:Number;
	// counter used to pause between ship regeneration
	private var shipCounter:Number;
	// counter used to pause while ship is in hyperspace
	private var hyperCounter:Number;
	// the number the hypercounter needs to run up to before bringing the ship out of hyperspace
	private var hyper:Number;
	// number of lives the player has
	private var numLives:Number;
	// the game state: true = game in progress, false: game is over
	private var state:Boolean;
	// the player score
	private var score:Number;
	// the frequency with which the ufo's appear
	private var ufoFreq:Number;
	// number used to switch every set of points to gain an extra life
	private var extra:Number;
	// define two new objects, the ufo and the spaceship, using the appropriate classes
	public var spaceShip:Ship;
	public var ufo:Ufo;
	//
	// getter and setter functions for the game
	public function setState(st):Void {
		state = st;
	}
	public function getState():Boolean {
		return state;
	}
	public function setScore(sc:Number):Void {
		score = sc;
	}
	public function getScore():Number {
		return score;
	}
	//
	// define the game object
	// numRoids = number of asterois to put on the board
	// numLives = number of lives
	// target = movie clip to attach game assets to
	public function Game(numRoids:Number, numLives:Number, target:MovieClip) {
		// initialize all game variables (defined above)
		this.bulletArray = new Array();
		this.roidArray = new Array();
		this.bulletCounter = 0;
		this.bulletPause = 0;
		this.roidCounter = 0;
		this.ufoCounter = 0;
		this.shipCounter = 0;
		this.hyperCounter = 0;
		this.extra = 1;
		// pull in constants to the game object (there has to be a better cleaner way to do this, so I think this code is sloppy)
		this.maxBulletPause = _root.SHIPBULLETPAUSE;
		this.deadZone = _root.ASTEROIDDEADZONE;
		this.hyper = _root.HYPER;
		// initialize game parameters
		this.numRoids = numRoids;
		this.numLives = numLives;
		this.ufoFreq = _root.UFOFREQUENCY;
		// define mc to attach game to
		this.target = target;
		// update the lives left icons in the upper left corner
		updateLives(0);
		// update the scoreboard
		updateScore(0);
		// the the game state to playing (true)
		setState(true);
		// set the player score to 0
		setScore(0);
	}
	// function to initialize the spaceship
	public function initSpaceShip():Void {
		// add new spaceShip object
		spaceShip = new Ship(Stage.width/2, Stage.height/2, target, _root.SHIPLEVEL);
	}
	// function to put a ufo on the board
	public function initUfo():Void {
		// choose a random y position
		var tempY:Number = (Math.random()*Stage.height)/2;
		// place the ufo either high up or low down on the board, but not in the center as that would make the game unfair
		tempY += (tempY>(Stage.height/4))*(Stage.height/2);
		// choose a random number to determine whether the ufo is travelling from right ot left or left to right
		var tempType:Number = int(Math.random()*2);
		if (Math.random()>.5) {
			// move from left to right
			var tempX:Number = 0;
			var startAngle:Number = 90;
		} else {
			// move from right to left
			var tempX:Number = Stage.width;
			var startAngle:Number = 270;
		}
		// add new ufo object
		ufo = new Ufo(tempX, tempY, startAngle, _root.UFOSPEED, tempType, target, _root.UFOLEVEL);
	}
	// function to reset the board
	// called when a board is cleared of all asteroids
	public function resetBoard():Void {
		// add two to the number of asteroids, so that each level gets progressivley harder
		numRoids += 2;
		// multiply the frequency of ufo so the next level has more ufos occuring
		ufoFreq *= 1.2;
		// initialize the asteroid field
		initRoids(numRoids);
	}
	// function to initialize the asteroid field
	public function initRoids(num:Number):Void {
		// loop and initialize asteroids
		for (var i = 0; i<num; ++i) {
			// choose a random scale (50%, 75% and 100%)
			var tempscale:Number = (Math.round(Math.random()*2)*25)+50;
			// choose a random x and y position ouside of the deadzone
			var tempX:Number = (Math.random()*Stage.width)-deadZone;
			var tempY:Number = (Math.random()*Stage.height)-deadZone;
			if (tempX>spaceShip.getX()-(deadZone/2)) {
				tempX += deadZone;
			}
			if (tempY>spaceShip.getY()-(deadZone/2)) {
				tempY += deadZone;
			}
			// add asteroid to the asteroid array and initialize each
			roidArray.push(new Asteroid(tempX, tempY, tempscale, target, i+_root.ASTEROIDLEVEL));
		}
		// set up the asteroid counter to the current number of asteroids
		roidCounter = num;
	}
	// function to explode roid
	public function explodeRoid(roid:Object):Void {
		// first, determine the current scale of the asteroid
		var cScale:Number = (roid.getScale()/25)-2;
		// if the asteroid is above 50%, then break the asteroid into two smaller asteroids
		if (cScale>0) {
			// determine the new size of each of the new asteroids by taking the size of the one hit, and scaling it down (-25%)
			var newScale:Number = ((cScale-1)*25)+50;
			// add the two new asteroids to the asteroids array
			for (var i = 0; i<2; ++i) {
				roidArray.push(new Asteroid(roid.getClip()._x, roid.getClip()._y, newScale, target, roidCounter+_root.ASTEROIDLEVEL));
				++roidCounter;
			}
		} else {
			// if the asteroid is at 50% already (its smallest), then just blow it up without dividing it
			roid.setState(1);
			roid.getClip().gotoAndPlay(5);
		}
	}
	// function to dismantle all bullets currently moving across the screen by removing them
	public function killAllBullets():Void {
		for (var i in bulletArray) {
			bulletArray[i].kill();
		}
		// re-initialize the bullet array
		bulletArray = new Array();
		// reset the bullet counter
		bulletCounter = 0;
		// turn off the firing flag
		target.firing = false;
	}
	// update the bullet counter so that all new bullets are placed on an unconflicting level
	public function updateBulletCounter():Void {
		++bulletCounter;
		// if the counter is greater than 2000 (way more than needed), reset the counter
		if (bulletCounter>2000) {
			bulletCounter = 0;
		}
	}
	// function fire the bullet
	// entity: variable used to determine whether the bullet was fired from the spaceship or the ufo
	public function fireBullet(entity:String):Void {
		// if the bullet was fired from the ufo
		if (entity == "ufo") {
			// first check to see if there has been enough time between the last bullet fired from the ufo
			if (ufo.getBulletPause() == 0) {
				// if so, fire!
				// set up tracking so the ufo bullet is heading from the ufo to the spaceship (aiming)
				var xdiff:Number = spaceShip.getX()-ufo.getX();
				var ydiff:Number = spaceShip.getY()-ufo.getY();
				// determine the angle of the bullet from the angle between the ship and ufo
				var tempAngle:Number = Math.atan2(ydiff, xdiff);
				// convert the angle to radians
				tempAngle = tempAngle*180/Math.PI;
				// add new bullet to the master bullet array
				bulletArray.push(new Bullet(entity, ufo.getX(), ufo.getY(), 0, 0, ufo.getBulletSpeed(), tempAngle+90, target, bulletCounter+_root.BULLETLEVEL));
				// update the bullet counter
				updateBulletCounter();
			}
			// update the pause counter between the bullets from the ufo
			ufo.updateBulletPause();
			// if the counter has reached its maximum, then the ufo is ready to fire another bullet
			if (ufo.getBulletPause()>ufo.getMaxBulletPause()) {
				ufo.setBulletPause(0);
			}
		} else {
			// if the bullet was fired from the ship
			// first make sure the ship is not blowing up or in hyper space
			if (spaceShip.getState() == 0) {
				// add new bullet to the bullet array
				bulletArray.push(new Bullet("ship", spaceShip.getX(), spaceShip.getY(), spaceShip.getXV(), spaceShip.getYV(), spaceShip.getBulletSpeed(), spaceShip.getAngle(), target, bulletCounter+_root.BULLETLEVEL));
				// update the bullet counter
				updateBulletCounter();
			}
		}
	}
	// set up the score box and game over messages in the playing area
	public function setUpUI():Void {
		// set up score box and position
		target.attachMovie("score_box", "score_box", _root.SCORELEVEL);
		target.score_box._x = 35;
		target.score_box._y = 35;
		// set up game over message and position
		target.attachMovie("text_game_over", "end", _root.ENDGAMELEVEL);
		target.end._visible = false;
		target.end._x = Stage.width/2;
		target.end._y = Stage.height/2;
	}
	// update the score with
	// addscore: amount to be added to the player score
	public function updateScore(addScore:Number):Void {
		setScore(getScore()+addScore);
		// update the score box in the playing area with the new score
		target.score_box.scoreText.text = getScore();
		// now check to see if the score has reached the extra life level
		if ((getScore()/_root.EXTRALIFE)>extra) {
			// if so, add one extra life
			++extra;
			updateLives(1);
		}
	}
	// check for all collisions between game elements (very important)
	// this function is by far the most taxing on the cpu
	// if there is any way to optimize this function, I'm open to suggestions
	public function checkForCollision():Void {
		// first, check to see if the spaceship has collided with the ufo
		if (spaceShip.getClip().hitTest(ufo.getX(), ufo.getY(), true) and spaceShip.getState() == 0 and ufo.getState() == 1) {
			// if so, give the player the points for the ufo kill
			updateScore((ufo.getType()+1)*_root.SCOREUFO);
			// destroy both the spaceship and the ufo
			spaceShip.destroy();
			ufo.destroy();
		}
		// next, run through all the asteroids and check each against any collision with the ufo, the ship or any bullets
		// bypass the entire procedure if there are no asteroids left
		if (roidArray.length>0) {
			// set up a bullet flag so that all the bullets are checked only once
			var bFlag:Boolean = false;
			// now loop through all asteroids
			for (var i in roidArray) {
				// set up flag so that if any asteroid is hit by anything, it only is deleted once
				var rflag:Boolean = false;
				// move the asteroid to its new location
				roidArray[i].moveAsteroid();
				// check to see if the asteroid has hit the spaceship
				if (roidArray[i].getClip().hitTest(spaceShip.getX(), spaceShip.getY(), true) and spaceShip.getState() == 0) {
					// if so, destroy the spaceship and blow up the asteroid
					spaceShip.destroy();
					explodeRoid(roidArray[i]);
					// set the asteroid flag so that it is deleted once
					rflag = true;
				}
				// check to see if the asteroid has collided with the ufo
				if (roidArray[i].getClip().hitTest(ufo.getX(), ufo.getY(), true) and ufo.getState() == 1) {
					// destroy the ufo and the asteroid
					ufo.destroy();
					explodeRoid(roidArray[i]);
					// set the asteroid flag so that it is deleted once
					rflag = true;
				}
				// now iterate through all the bullets to see if they have hit anything
				for (var k in bulletArray) {
					// set up flag so that if a buulet has hit anything, it will be deleted once
					var tFlag:Boolean = false;
					// check the bullet flag so that the bullets are moved once, not as many times as there are asteroids in the playing area
					// this is really a hack to attempt to optimize the number of nested loops the game has to process, to save cpu cycles
					if (!bFlag) {
						// move the bullet
						bulletArray[k].moveBullet();
					}
					// check to see if any bullet has hit any asteroid
					if (roidArray[i].getClip().hitTest(bulletArray[k].getX(), bulletArray[k].getY(), true) and bulletArray[k].getState() == 1) {
						// if yes, then set up the flag so that the asteroid gets deleted
						rflag = true;
						// destroy the asteroid
						explodeRoid(roidArray[i]);
						// set the flag so the bullet is deleted from the array
						tFlag = true;
						// if the bullet was fired from the ship, then update the user score
						if (bulletArray[k].getEntity() == "ship") {
							// determine the amount to add to the user score
							// the smaller the asteroid, the more points are added to the score
							updateScore(150-roidArray[i].getScale());
						}
					}
					// check to see if a bullet has hit the ufo AND the bullet was fired from the spaceship, not itself, AND if the ufo is not exploding
					if (ufo.getClip().hitTest(bulletArray[k].getX(), bulletArray[k].getY(), true) and bulletArray[k].getState() == 1 and ufo.getState() == 1 and bulletArray[k].getEntity() == "ship") {
						// update the score with a score that is determined by the size of the ufo
						// the smaller the ufo, the more points awarded
						updateScore((ufo.getType()+1)*_root.SCOREUFO);
						// desstroy the ufo
						ufo.destroy();
						// set the flag so the bullet is deleted from the array
						tFlag = true;
					}
					// check to see if a bullet has hit the spaceship AND the bullet was fired from the ufo, not itself, AND if the spaceship is not exploding or in hyperspace
					if (spaceShip.getClip().hitTest(bulletArray[k].getX(), bulletArray[k].getY(), true) and bulletArray[k].getState() == 1 and spaceShip.getState() == 0 and bulletArray[k].getEntity() == "ufo") {
						// destory the spaceship
						spaceShip.destroy();
						// set the flag so the bullet is deleted from the array
						tFlag = true;
					}
					// now check to see if the bullet has gone off the edge of the screen
					if (bulletArray[k].getX()>Stage.width+10 or bulletArray[k].getX()<-10 or bulletArray[k].getY()>Stage.height+10 or bulletArray[k].getY()<-10) {
						// if so, then set the flag so the bullet is deleted from the array
						tFlag = true;
					}
					// now if the flag has been set, remove the bullet from the bullet array and remove the graphic from the stage
					if (tFlag) {
						bulletArray[k].kill();
						bulletArray.splice(Number(k), 1);
					}
				}
				// now that the bullets have been iterated once through, do not move them further within this loop cycle
				bFlag = true;
				// now check and see if the asteroid has been hit with anything or not
				if (rflag) {
					// if so, remove it from the asteroid array and destroy it
					// this work because when an asteroid of 75 or 100% is hit and divides, essentially two new asteroids are created and the original is destroyed and deleted
					roidArray[i].destroy();
					roidArray.splice(Number(i), 1);
				}
			}
		} else {
			// if there are NO asteroids left to loop through, then the board/level has been cleared
			// the board now needs to be reset for the next level
			resetBoard();
		}
	}
	// this function was added so that after the spaceship has been destroyed and the appropriate pause has passed, the game will "intelligently" wait until the cenetr area of the screen is clear of all bullets, asteroids, and ufos before it respawns the spaceship
	public function respawn():Boolean {
		// set the allclear flag
		var allClear:Boolean = true;
		// now iterate through all asteroids to see if ANY are in the deadzone area around the ship
		for (var i in roidArray) {
			var tempX:Number = roidArray[i].getX();
			var tempY:Number = roidArray[i].getY();
			var shipX:Number = Stage.width/2;
			var shipY:Number = Stage.height/2;
			var diff:Number = deadZone/2;
			// if ANY are in the area, even one, then the board is not clear, and therefore not ready for a new spaceship 
			if (tempX>shipX-diff and tempX<shipX+diff and tempY>shipY-diff and tempY<shipY+diff) {
				allClear = false;
			}
		}
		// also check to see if any ufos are in the area.
		// if a ufo is in the area, the board is not ready for a new spaceship
		if (ufo.getState()>0) {
			allClear = false;
		}
		return allClear;
	}
	// update the number of lives left by some amount:
	// diff: number to add or subtract from the number of lives left
	public function updateLives(diff:Number):Boolean {
		numLives += diff;
		// first, clear all life icons off the board
		for (var i = 0; i<_root.MAXLIVES; ++i) {
			target["life"+i].removeMovieClip();
		}
		// now put as many icons on the baord as there are lives left
		for (var i = 0; i<numLives; ++i) {
			// atach the correct graphic to the board at the correct level
			target.attachMovie("shipgraphic", "life"+i, _root.LIVESLEVEL+i);
			// place in the right position
			target["life"+i]._x = 50+(i*15);
			target["life"+i]._y = 80;
			target["life"+i]._alpha = 50;
		}
		// return a boolean to the calling function so that it knows immediately whether there are any lives left, or the game is over
		if (numLives<0) {
			return false;
		} else {
			return true;
		}
	}
	// function the completely clear the board so that the game can be replayed
	public function endGame():Void {
		// first off, delete every single object on the stage, including constants, movie clips and functions
		for (var i in target) {
			target[i].removeMovieClip();
			delete target[i];
		}
		// now delete all items from the asteroid array
		// this may be overkill, but I ran into bugs here so this is just a safety precaution
		for (var i in roidArray) {
			roidArray.splice(Number(i), 1);
		}
		// re initialize the asteroid array
		roidArray = new Array();
		// delete all other game objects
		delete roidArray;
		delete spaceShip;
		delete ufo;
		delete target;
	}
	// function that updates the game every frame cycle
	public function update(torque:Number, force:Number, hyperFlag:Boolean):Void {
		// determine if the game is still in progress, or over
		if (state) {
			// if the game is in progress, turn off the game over text box
			target.end._visible = false;
			// check to see if the spaceship is exploding
			if (spaceShip.getState() == 1) {
				// if so, start the counter so that there is at least a 2 second pause before a ship respawn
				++shipCounter;
				// if the correct amount of time has passed...
				if (shipCounter>_root.SHIPRESPAWNDELAY) {
					// ... AND the area is clear and ready...
					if (respawn()) {
						// ... AND there are lives left...
						if (updateLives(-1)) {
							// ...reset the ship
							shipCounter = 0;
							// delete all bullets off the stage
							killAllBullets();
							// add a new ship
							initSpaceShip();
						} else {
							// no more lives left, end the game
							state = false;
						}
					}
				}
			} else {
				// if the spaceship is NOT exploding, check to see if the ship is in hyperspace
				if (hyperFlag or hyperCounter>0) {
					// place in hyperspace
					spaceShip.hyper();
					// update the counter
					++hyperCounter;
					// if the spaceship has been in hyperspace for the full length of time, bring it out of hyperspace
					if (hyperCounter>hyper) {
						// reset the counter
						hyperCounter = 0;
						// set the ship state back to active
						spaceShip.setState(0);
						// pick a random spot on the stage for the ship
						spaceShip.setX(Math.random()*Stage.width);
						spaceShip.setY(Math.random()*Stage.height);
						spaceShip.x = spaceShip.getX();
						spaceShip.y = spaceShip.getY();
					}
				} else {
					// if the spaceship is not in hyperspace, and not blowing up, then simply move it according to any force and torque applied to it
					spaceShip.update(torque, force);
				}
			}
			// randomly check to see if a ufo should be launch by checking a random number against the ufo frequency, and checking to make sure the ufo is not already crossing the screen or exploding
			if (Math.random()<ufoFreq and (ufo.getState() == 0 or ufo.getState() == undefined) and spaceShip.getState() == 0) {
				// add a ufo to the stage
				initUfo();
			}
		} else {
			// if the state of the game is false, then the game is over
			// show the game over message
			target.end._visible = true;
		}
		// if the ufo is on the screen, and the ship is not exploding or in hyperspace, then put in a request to have the ufo fire a bullet at the ship
		if (ufo.getState() == 1) {
			if (spaceShip.getState() == 0) {
				fireBullet("ufo");
			}
			// if the ufo is on the screen, update its position
			ufo.moveUfo();
		} else if (ufo.getState() == 2) {
			// if the ufo is exploding, update the ufo counter
			// this allows enough time for the ufo to finish exploding before another is initialized
			++ufoCounter;
			if (ufoCounter>10) {
				ufoCounter = 0;
				// when a ufo state is set to 0, that means it is not currently doing anything and is ready to be respawned
				ufo.setState(0);
			}
		}
		// now check for all collisions
		checkForCollision();
	}
	// function to initialize the game
	public function init():Void {
		// first, set up a new spaceship
		initSpaceShip();
		// set up all the asteroids
		initRoids(numRoids);
		// set up all text fields
		setUpUI();
	}
}
