﻿// Ship class
//
class Ship {
	// define all public and private variables used in the class
	// the movie clip associated with the ship
	private var ship_mc:MovieClip;
	// the x and y position
	public var x:Number;
	public var y:Number;
	// the acceleration along the x and y axis
	private var xa:Number;
	private var ya:Number;
	// the velocity along the x and y axis
	private var xv:Number;
	private var yv:Number;
	// general velocity, used to determine drag
	private var v:Number;
	// viscosity used to help slow the ship down when it is being carried by momentum
	private var visc:Number;
	// the amount of drag on the ship
	private var drag:Number;
	// contains the current angle of rotation for the ship
	private var angle:Number;
	// the velocity at which the ship is rotating
	private var AngVelocity:Number;
	// the acceleration at which the ship is rotating
	private var AngAcceleration:Number;
	// the state of the ship
	private var state:Number;
	// a flag used to determine whether the ship thrust sound is playing or not, to avoid overlapping sounds
	private var soundPlaying:Boolean;
	// the speed at which the bullet is fired
	private var bulletSpeed:Number;
	// getter/setter functions
	public function getX():Number {
		return ship_mc._x;
	}
	public function setX(x:Number):Void {
		ship_mc._x = x;
	}
	public function getY():Number {
		return ship_mc._y;
	}
	public function setY(y:Number):Void {
		ship_mc._y = y;
	}
	public function getXV():Number {
		return xv;
	}
	public function getYV():Number {
		return yv;
	}
	public function setXV(sxv:Number):Void {
		xv = sxv;
	}
	public function setYV(syv:Number):Void {
		yv = syv;
	}
	public function getAngle():Number {
		return ship_mc._rotation;
	}
	public function setAngle(ang:Number):Void {
		ship_mc._rotation = ang;
	}
	public function getState():Number {
		return state;
	}
	public function setState(st:Number):Void {
		state = st;
	}
	public function getBulletSpeed():Number {
		return bulletSpeed;
	}
	public function setBulletSpeed(bs:Number):Void {
		bulletSpeed = bs;
	}
	public function getClip():MovieClip {
		return ship_mc;
	}
	//
	// define the ship object
	// x: starting x location
	// y: starting y location
	// target: target movie clip to attach ship to
	// depth: level the ship will be placed on
	public function Ship(x:Number, y:Number, target:MovieClip, depth:Number) {
		// set up initial variables (defined above)
		this.xa = 0;
		this.ya = 0;
		this.xv = 0;
		this.yv = 0;
		this.v = 0;
		this.visc = 0.001;
		this.drag = 0;
		this.angle = 0;
		this.AngVelocity = 0;
		this.AngAcceleration = 0;
		this.soundPlaying = false;
		// pass on all params to the ship object properties
		// set the x and y position
		this.x = x;
		this.y = y;
		// atach the ship graphic to the ship object
		ship_mc = target.attachMovie("shipgraphic", "ship", depth);
		// attach the ship thrust sound to the ship movie clip
		ship_mc.thrust = new Sound(ship_mc);
		ship_mc.thrust.attachSound("thrust");
		// attach the ship explosion sound to the ship movie clip
		ship_mc.explode = new Sound(ship_mc);
		ship_mc.explode.attachSound("shipexplode");
		// set the x and y position
		setX(x);
		setY(y);
		// set its state to 0 (normal)
		setState(0);
		// set up the bullet speed
		setBulletSpeed(_root.SHIPBULLETSPEED);
	}
	// function to turn the ship
	// i found this code and used it, but i tweaked it so that the amount of angualr acceleration is so low that the ship appears to turn left or right instantaneously
	private function turnShip(torque:Number):Void {
		// determine the speed at which the ship is turning, if any
		var rtorque:Number = 0.5*AngVelocity;
		AngAcceleration = torque-rtorque;
		AngVelocity += AngAcceleration;
		// adjust the ship angle with the angular acceleration
		angle += AngVelocity;
		// make sure tha angle does not go over 360 degrees or under 0 degrees
		if (angle<0) {
			angle += 360;
		}
		if (angle>360) {
			angle -= 360;
		}
		// set the new angle of the ship
		setAngle(angle);
	}
	// function to move the ship in the direction it is pointing if force is applied to it
	private function moveShip(force:Number):Void {
		// if any force is being applied to the ship
		if (force) {
			// first, check to see if the thrust sound is playing
			// if not...
			if (!soundPlaying) {
				// ... play the thrust sound and set the ship graphic so that it looks like it is rocketing
				ship_mc.gotoAndPlay(2);
				ship_mc.thrust.start(0, 1000);
				// set the flag so that there are no sound overlaps
				soundPlaying = true;
			}
		} else {
			// if ship has no force being applied to it
			if (state == 0) {
				// turn off the thrust sound and set the ship graphic so that there is no afterburner
				ship_mc.gotoAndStop(1);
				ship_mc.thrust.stop();
			}
			// set the flag so that the sound can be played again
			soundPlaying = false;
		}
		// determine the velocity
		v = Math.sqrt((xv*xv)+(yv*yv));
		// determine how much drag is on the ship
		drag = visc*v;
		// apply the drag to the equation to determine how much x and y acceleration there is
		xa = ((drag*xv)*-1)+(force*Math.sin(Math.PI/180*angle));
		ya = ((drag*yv)*-1)-(force*Math.cos(Math.PI/180*angle));
		// add the acceleration to the x and y velocities
		xv += xa;
		yv += ya;
		// add the x and y velocities to the x and y positions
		y += yv;
		x += xv;
		// check to see if the ship has gone off the end of the stage
		// if the ship runs off the edge of the stage, it is placed directly across from its last know position on the stage
		if (x>Stage.width+10) {
			x = -10;
		} else if (x<-10) {
			x = Stage.width+10;
		}
		if (y>Stage.height+10) {
			y = -10;
		} else if (y<-10) {
			y = Stage.height+10;
		}
		// update the ship movie clip with the new x and y position
		setX(x);
		setY(y);
	}
	// catch all function that updates the location and rotation of the ship
	public function update(torque:Number, force:Number):Void {
		// apply torque to the angular acceleration of the ship
		turnShip(torque);
		// apply force to the ship in the direction it is facing
		moveShip(force);
	}
	// function to hide the ship while it is in hyperspace
	public function hyper():Void {
		// set the stse to hyperspace
		setState(2);
		// move the ship so far off the screen so as to appear invisible
		setX(-1000);
		setY(-1000);
	}
	// function to destroy the ship
	public function destroy():Void {
		// stop the thrust sound and start the explsion sound
		ship_mc.thrust.stop();
		ship_mc.explode.start();
		// set the ship state to the eploding state
		setState(1);
		// go to and play the exploding animation in the ship movie clip
		ship_mc.gotoAndPlay(5);
	}
	// function to reset the ship to its starting state and location after it has hyperspaced
	public function reset(x:Number, y:Number):Void {
		// reset the state
		setState(0);
		// move the ship to its new x, y position
		setX(x);
		setY(y);
		// stop the ship's movement
		setXV(0);
		setYV(0);
		// reset the ship angle
		setAngle(0);
		// reset the ship movie clip back to its normal view (non thrusting and non blowing up)
		ship_mc.gotoAndStop(1);
	}
	// function to removie the ship movie clip from the stage
	public function kill():Void {
		ship_mc.removeMovieClip();
	}
}
