﻿// Ufo class
//
class Ufo {
	// define all public and private variables used in the class
	// the movie clip associated with the ufo
	private var ufo_mc:MovieClip;
	// the x and y position
	private var x:Number;
	private var y:Number;
	// the velocity along the x and y axis
	private var xv:Number;
	private var yv:Number;
	// the vector speed at which the ufo travels in
	private var speed:Number;
	// contains the current angle of rotation for the ufo
	private var angle:Number;
	// the state of the ufo
	private var state:Number;
	// the type of the ufo (0=large, 1=small)
	private var ufoType:Number;
	// a flag used to determine whether the ufo sound is playing or not, to avoid overlapping sounds
	private var soundPlaying:Boolean;
	// array to conatin the different sounds the ufo can make
	private var soundArray:Array;
	// the counter used to determine when another ufo bullet can be fired
	private var bulletPause:Number;
	// the maximum amount of time between ufo bullet firings
	private var maxBulletPause:Number;
	// the speed at which bullets fired from the ufo move
	private var bulletSpeed:Number;
	// getter/setter functions
	public function getX():Number {
		return ufo_mc._x;
	}
	public function setX(x:Number):Void {
		ufo_mc._x = x;
	}
	public function getY():Number {
		return ufo_mc._y;
	}
	public function setY(y:Number):Void {
		ufo_mc._y = y;
	}
	public function getXV():Number {
		return xv;
	}
	public function getYV():Number {
		return yv;
	}
	public function getAngle():Number {
		return angle;
	}
	public function setAngle(ang:Number):Void {
		angle = ang;
	}
	public function getState():Number {
		return state;
	}
	public function setState(st:Number):Void {
		state = st;
	}
	public function getType():Number {
		return ufoType;
	}
	public function setType(t:Number):Void {
		ufoType = t;
	}
	public function getMaxBulletPause():Number {
		return maxBulletPause;
	}
	public function setMaxBulletPause(mbp:Number):Void {
		maxBulletPause = mbp;
	}
	public function getBulletPause():Number {
		return bulletPause;
	}
	public function setBulletPause(bp:Number):Void {
		bulletPause = bp;
	}
	public function getBulletSpeed():Number {
		return bulletSpeed;
	}
	public function setBulletSpeed(bs:Number):Void {
		bulletSpeed = bs;
	}
	public function getClip():MovieClip {
		return ufo_mc;
	}
	public function updateBulletPause():Void {
		++bulletPause;
	}
	//
	// define the ufo object
	// x: starting x location
	// y: starting y location
	// angle: angle at which the ufo is flying in
	// speed: speed at which the ufo moves
	// type: the type of ufo (large or small)
	// target: target movie clip to attach ship to
	// depth: level the ship will be placed on
	public function Ufo(x:Number, y:Number, angle:Number, speed:Number, type:Number, target:MovieClip, depth:Number) {
		// set up initial variables (defined above)
		this.xv = 0;
		this.yv = 0;
		this.soundPlaying = false;
		this.soundArray = new Array("LSAUCER", "SSAUCER");
		// pass on all params to the ufo object properties
		// set the x and y position
		this.x = x;
		this.y = y;
		this.angle = angle;
		this.speed = speed;
		setType(type);
		// attach the correct ufo graphic to the ufo object
		ufo_mc = target.attachMovie("ufographic", "ufo", depth);
		// attach the thrust sound to the ufo movie clip
		// use a different sound depending on whether the ufo is small or large
		ufo_mc.thrust = new Sound(ufo_mc);
		ufo_mc.thrust.attachSound(soundArray[type]);
		// attach the ufo explode sound to the ufo movie clip
		ufo_mc.explode = new Sound(ufo_mc);
		ufo_mc.explode.attachSound("shipexplode");
		// start the ufo thrust sound
		ufo_mc.thrust.start(0, 1000);
		// set the scale of the ufo based on the type variable
		ufo_mc._xscale = (100-(type*50));
		ufo_mc._yscale = ufo_mc._xscale;
		// set the x and y positions
		setX(x);
		setY(y);
		// set the ufo state to 1 (on)
		setState(1);
		// reset the bullet pause (counter) to 0
		setBulletPause(0);
		// set the bullet speed
		setBulletSpeed(_root.UFOBULLETSPEED);
		// set the maximum bullet pause for the bullet pause counter
		setMaxBulletPause(_root.UFOBULLETPAUSE);
	}
	// function to move the ufo
	public function moveUfo():Void {
		// first, check to see if the ufo is moving left to right
		if (angle == 90) {
			// if so, check to see if the ufo is anywhere in the vicinity of left quadrant line (25% of the way across the screen)
			if (Math.abs(x-(Stage.width*.25))<5) {
				// if so, make a small random guess to adjust the direction of the ufo (50% chance)
				if (Math.random()>.5) {
					// if so, check to see if the ufo is near the top of the screen or the bottom
					if (y>(Stage.height/2)) {
						// if near the bottom of the screen, change the angle to 45 degrees
						angle = 45;
					} else {
						// if near the top, change the angle of the ufo to 135 degrees
						angle = 135;
					}
				}
			}
			// or is the ufo travelling from right to left
		} else if (angle == 270) {
			// if so, check to see if the ufo is anywhere in the vicinity of right quadrant line (75% of the way across the screen)
			if (Math.abs(x-(Stage.width*.75))<5) {
				// if so, make a small random guess to adjust the direction of the ufo (50% chance)
				if (Math.random()>.5) {
					// if so, check to see if the ufo is near the top of the screen or the bottom
					if (y>(Stage.height/2)) {
						// if near the bottom of the screen, change the angle to 315 degrees
						angle = 315;
					} else {
						// if near the top, change the angle of the ufo to 225 degrees
						angle = 225;
					}
				}
			}
			// or if the ufo is already travelling diagonally across the screen, still from left to right...
		} else if (angle == 45 or angle == 135) {
			// if so, check to see if the ufo is anywhere in the vicinity of right quadrant line (75% of the way across the screen)
			if (Math.abs(x-(Stage.width*.75))<5) {
				// so now that it's travelled 3/4 of the way across the screen and at an angle, set the angle back to horizontal
				angle = 90;
			}
			// or if the ufo is already travelling diagonally across the screen, still from right to left...
		} else if (angle == 315 or angle == 225) {
			// if so, check to see if the ufo is anywhere in the vicinity of left quadrant line (25% of the way across the screen)
			if (Math.abs(x-(Stage.width*.25))<5) {
				// so now that it's travelled 3/4 of the way across the screen and at an angle, set the angle back to horizontal
				angle = 270;
			}
		}
		// update the x, y velocities of the ufo by its speed and angle
		xv = speed*Math.sin(Math.PI/180*angle);
		yv = -speed*Math.cos(Math.PI/180*angle);
		// update the x, y position of the ufo
		y += yv;
		x += xv;
		// test to see if the ufo has gone off the far left or right of the screen
		if (x>Stage.width+10 or x<-10 or y>Stage.height+10 or y<-10) {
			// if so, kill it
			kill();
		}
		// update the x, y position of the ufo movie clip
		setX(x);
		setY(y);
	}
	// function to remove the ufo movie clip from the stage
	public function kill():Void {
		// set the state of the ufo to non-active
		setState(0);
		// turn off the ufo sound
		ufo_mc.thrust.stop();
		// removie the ufo movie clip from the stage
		ufo_mc.removeMovieClip();
	}
	// function to destroy the ufo
	public function destroy():Void {
		// stop the thrust sound and start the explosion sound
		ufo_mc.thrust.stop();
		ufo_mc.explode.start();
		// set the ufo state to 2 (exploding)
		setState(2);
		// play the exploding ufo animation in the ufo movie clip
		ufo_mc.gotoAndPlay(2);
	}
}
